"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A Graph is a non-linear data structure consisting of nodes and edges.
 *
 * The nodes are sometimes also referred to as vertices and the edges are lines or arcs that connect any two nodes in the graph.
 *
 * ! Important: This is an impleamentation of directed graph
 */
var Graph = /** @class */ (function () {
    /**
   * A Graph is a non-linear data structure consisting of nodes and edges.
   *
   * The nodes are sometimes also referred to as vertices and the edges are lines or arcs that connect any two nodes in the graph.
   *
   * ! Important: This is an impleamentation of directed graph
   *
   * @constructor
   * Initialize graph with a new Map(Adjacency List) of type K,K[]
   */
    function Graph() {
        this.adList = new Map();
    }
    /**
     * @property returns Adjacency List
    */
    Graph.prototype.getAdlist = function () {
        return this.adList;
    };
    /**
     * add Node to Adjacency List with an empty array
     * @param node
     */
    Graph.prototype.addNode = function (node) {
        this.adList.set(node, []);
    };
    /**
     * add an edge to graph going from origin to destination
     * @param origin start Node
     * @param destination destination Node
     */
    Graph.prototype.addEdge = function (origin, destination) {
        this.adList.get(origin).push(destination);
    };
    /**
     * The Breadth First Search (BFS) traversal is an algorithm, which is used to visit all of the nodes of a given graph.
     *
     * In this traversal algorithm one node is selected and then all of the adjacent nodes are visited one by one.
     * @param start starting Node
     * @param find Node that is to be found
     * @returns Resultant Array containing all nodes traversed from origin to result or false to tell that bfs failed to find the connection
     */
    Graph.prototype.bfs = function (start, find) {
        var visited = new Set();
        var result = [start];
        var queue = [start];
        while (queue.length > 0) {
            var origin = queue.shift(); // mutates the queue
            var destinations = this.adList.get(origin);
            for (var _i = 0, destinations_1 = destinations; _i < destinations_1.length; _i++) {
                var destination = destinations_1[_i];
                if (destination === find) {
                    result.push(destination);
                    return result;
                }
                if (!visited.has(destination)) {
                    visited.add(destination);
                    result.push(destination);
                    queue.push(destination);
                }
            }
        }
        return false;
    };
    /**
     * In this algorithm, one starting vertex is given, and when an adjacent vertex is found,
     * it moves to that adjacent vertex first and tries to traverse in the same manner.
     *
     * TODO: redo dfs for correct behavior
     * @param start starting Node
     * @param find final Node
     */
    Graph.prototype.dfs = function (start, find) {
    };
    return Graph;
}());
exports.default = Graph;
;
//# sourceMappingURL=Graph.js.map