"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A hash table is a data structure that is used to store keys/value pairs.
 *
 *  It uses a hash function to compute an index into an array in which an element will be inserted or searched.
 *
 * Need to specify the type for value in typescript
 */
var HashTable = /** @class */ (function () {
    /**
   * A hash table is a data structure that is used to store keys/value pairs.
   *
   *  It uses a hash function to compute an index into an array in which an element will be inserted or searched.
   *
   * @constructor Initialize Hash Table with certain memory size
   *
   * @param size numeric value for initializing hash table
   */
    function HashTable(size) {
        this.data = new Array(size);
    }
    /**
     * @private function that helps in getting a hash value
     * @param key string key that will be used for getting a numeric value
     */
    HashTable.prototype._hash = function (key) {
        var hash = 0;
        for (var i = 0; i < key.length; i++) {
            hash = (hash + key.charCodeAt(i) * i) % this.data.length;
        }
        return hash;
    };
    /**
     * Add key/value pair to the hash table
     *
     * Time complexity-> O(1)
     * @param key key of type string
     * @param value value of type K
     */
    HashTable.prototype.set = function (key, value) {
        var address = this._hash(key);
        if (!this.data[address]) {
            this.data[address] = [];
        }
        this.data[address].push({ key: key, value: value });
    };
    /**
     * get the value of the key/value pair in hash table
     * @param key of type string; used to get value from hash table
     *
     * @returns value of type K if found else false
     */
    HashTable.prototype.get = function (key) {
        var address = this._hash(key);
        var currentBucket = this.data[address];
        for (var _i = 0, currentBucket_1 = currentBucket; _i < currentBucket_1.length; _i++) {
            var bucket = currentBucket_1[_i];
            if (bucket.key === key) {
                return bucket.value;
            }
        }
        return false;
    };
    /**
     * @returns all the keys of hash table as a string array
     */
    HashTable.prototype.keys = function () {
        var keys = [];
        for (var _i = 0, _a = this.data; _i < _a.length; _i++) {
            var bucket = _a[_i];
            if (bucket) {
                for (var _b = 0, bucket_1 = bucket; _b < bucket_1.length; _b++) {
                    var ind = bucket_1[_b];
                    keys.push(ind.key);
                }
            }
        }
        return keys;
    };
    return HashTable;
}());
exports.default = HashTable;
//# sourceMappingURL=HashTable.js.map